package jess;

import java.util.*;
import java.io.*;

/** **********************************************************************
 * Class used to represent deffacts. Note that you can create Deffacts objects
 * and add them to a Rete engine using Rete.addDeffacts().
 * <P>
 * (C) 1998 E.J. Friedman-Hill and the Sandia Corporation
 * @author Ernest J. Friedman-Hill
 ********************************************************************** */

public class Deffacts implements Serializable {
    
    private String m_name;
    private ArrayList m_facts;
    private String m_docstring = "", m_string = "";

    /**
     * Fetch the name of thie deffacts
     * @return the name
     */
    public final String getName() { return m_name; }

    /**
     * Fetch the documentation comment, if any, for this deffacts
     * @return the documentation string
     */
    public final String getDocstring() { return m_docstring; }

    /**
     * Create a deffacts
     * @param name The name of the deffacts
     * @param docstring A documentation string
     */

    public Deffacts(String name, String docstring) {
        m_name = name;
        m_facts = new ArrayList();
        m_docstring = docstring;
    }
    
    /**
     * Add a fact to this deffacts
     * @param fact The fact to add
     */
  
    public void addFact(Fact fact) {
        m_facts.add(fact);
    }
    
    /**
     * Fetch the number of facts in this deffacts
     * @return the number of facts
     */  

    public int getNFacts() { return m_facts.size(); }

    /**
     * Fetch a single Fact from this deffacts
     * @param idx the o-based index of the desired fact
     * @return the idx'th fact
     */  

    public ValueVector getFact(int idx) {
        return (ValueVector) m_facts.get(idx);
    }
    
    /**
     * Assert my facts into engine.
     */
    
    public void reset(Rete engine) throws JessException {    
        try {
            Context gc = engine.getGlobalContext();
            for (int j=0; j<m_facts.size(); j++) {         
                Fact f = (Fact) m_facts.get(j);
                f = f.expand(gc);
                engine.assert(f);
            }
        } catch (JessException re) {
            re.addContext("assert from deffacts " + m_name);
            throw re;
        }                
    }
    
    /**
     * Describe myself
     * @return A string representation of this deffacts
     */

    public String toString() {
        List l = new List("deffacts", m_name);
        l.addQuoted(m_docstring);
        
        for (Iterator e = m_facts.iterator(); e.hasNext();)
            l.add((Fact) e.next());

        return l.toString();
    }

}


